; dossys1/asm
;
	TITLE	'<DosPLUS II.A - SYS01/SYS>'
;
;	THIS SYSTEM IS LOW RESIDING (1C00-21FF).
;	IT CONTAINS THE FOLLOWING ROUTINES:
;
;		$JP2DOS - NORMAL DOS ENTRY
;		$DOSCMD - INTERPRET CMD & JP2DOS
;		$RETCMD - INTERPRET CMD & RETURN
;		$FINDEV - FIND DEVICE/DRIVE BY NAME
;		$CLRXIT - FORCE EXIT TO 'DOS READY'
;		$EVAL	- EVALUATE CMD LINE
;		$FSPEC	- FETCH DEVICE/FILE SPEC
;		$PARAM	- FETCH OPTIONAL PARAMETERS
;		$FEXT	- PLACE DEFAULT EXTENSION
;		$FDRV	- FETCH DRIVE SPEC
;		$FDEV	- FIND DEVICE BY NAME
;		$VALUE	- CONVERT NUMERIC VALUE
;		$LIB	- FETCH LIBRARY ADDRESS
;		$CLS	- CLEAR VIDEO SCREEN
	PAGE
;
;	Outside support equivalences
;
BRKVEC	EQU	300H		;break key vector
DCBTBL	EQU	100H		;device table
UCASE	EQU	31EH		;convert A to upper case
REGSAV	EQU	303H		;save registers
;
FLAGK1	EQU	0167H		;system flag #1
DATAK5	EQU	016FH		;interpreter pointer
FLAG1S	EQU	018CH		;system flag
RUNDCB	EQU	1A00H		;run/load DCB
RESUME	EQU	0000H		;return to EXIT from cmd
KDCB	EQU	0336H		;keyboard DCB
CHAINF	EQU	KDCB+5		;keyboard flags
OVLYH	EQU	0190H		;high overlay in memory
;
STACK	EQU	1A00H		;stack area start
KEYBUFF EQU	01AFH		;80 char input buffer
BUFFER	EQU	1B00H		;I/O buffer (system)
;
VDLINE	EQU	9		;display string to video
KBLINE	EQU	5		;get string from kboard
VDCHAR	EQU	8		;video character
HLDKEY	EQU	29		;hold key processor
SCROLL	EQU	27		;scroll prot video
ERROR	EQU	39		;post error message
SETBRK	EQU	3		;set break vector
RUN	EQU	85		;load/run program
BADF	EQU	1		;bad function code
SCF	EQU	37H		;SCF opcode
NDRV	EQU	12		;no drive avaliable
;
;	Local support equivalences
;
ETX	EQU	3		;end of text
CR	EQU	13		;carriage return
ETB	EQU	23		;clear to EOL
;
SVC	EQU	08H		;execute SVC
OVLY	EQU	18H		;load/exec overlay
;
	PAGE
;	Table of vectored routines
;
	ORG	1C00H		;low overlay
;
VECTORS DW	RETURN		;0 - return
	DW	JP2DOS		;1
	DW	DOSCMD		;2
	DW	RETCMD		;3
	DW	BOOT		;4
	DW	FINDEV		;5
	DW	CLRXIT		;6
	DW	EVAL		;7
	DW	FSPEC		;8
	DW	PARAM		;9
	DW	FEXT		;10
	DW	FDRV		;11
	DW	FDEV		;12
	DW	VALUE		;13
	DW	LIB		;14
	DW	CLS		;15
;
	PAGE
;	$CLRXIT - @ABORT vector, jump to prompt
;
CLRXIT	LD	HL,CHAINF	;@KI DCB +5
	RES	1,(HL)		;turn off chaining
	LD	HL,FLAGK1	;system flag #1
	RES	2,(HL)		;debug OFF
;
	LD	HL,KEYBUFF	;must clear buffer
	LD	(HL),CR 	;put terminator
	LD	(DATAK5),HL	;save nil pointer
;
	PAGE
;	$JP2DOS - Normal DOS entry level
;
JP2DOS	LD	SP,STACK	;RE-INIT
	LD	HL,RESUME	;return address ($EXIT)
	PUSH	HL		;to stack
	XOR	A
	SBC	HL,HL		;AHL = 0
	LD	B,A		;B = 0
	LD	A,SETBRK	;SVC #
	RST	SVC		;go SETBRK
	LD	A,SCROLL	;SVC #
	RST	SVC		;turn off scroll protect
	INC	B		;B = 1
	LD	A,HLDKEY	;SVC #
	RST	SVC		;turn on hold processor
;
	LD	A,(FLAGK1)	;SYSTEM FLAGS
	AND	5		;BIT0 0 & 2
	CP	4		;BIT 0 ONLY?
	JR	NZ,EXIT2	;go if no
	LD	(FLAG1S),A	;TRIGGER DEBUG <>0
;
;	Fetch next available command
;
EXIT2	LD	HL,(DATAK5)	;get string pointer
EXIT0	CALL	POSHL		;any input
	INC	HL		;bump pointer
	JR	NZ,EXIT0	;go till terminator
	CP	';'             ;logical term?
	JR	Z,RTCMD0	;execute if yes
;
;	Prompt & input command line
;
READY	LD	HL,CHAINF	;chaining flag
	BIT	1,(HL)		;chaining on?
	JR	NZ,READY1	;get input if yes
	LD	HL,PROMPT	;else display prompt
	LD	BC,PRLEN<8+ETB	;length + EOT byte
	LD	A,VDLINE	;SVC #
	RST	SVC		;call VDLINE
;
READY1	LD	HL,BUFFER	;system I/O buffer
	LD	B,79		;length to fetch
	LD	A,KBLINE	;SVC #
	RST	SVC		;call KBLINE
	RET	C		;nil input, done!
	LD	DE,KEYBUFF	;dos buffer
	LD	A,(HL)		;fetch first char
	CP	'/'             ;repeat last command?
	JR	NZ,READY2	;IF NOT
	LD	H,D
	LD	L,E		;old command line
;
READY2	PUSH	DE		;save buffer
	LD	BC,79		;count
	LDIR			;to buffer
	LD	A,CR		;force C/R
	LD	(DE),A		;to last char
	POP	HL		;get buffer
	LD	A,(HL)		;get char
	CP	'.'             ;remark?
	JR	NZ,RTCMD0	;if not
	RET			;done!
;
	PAGE
;	$DOSCMD - Interpret cmd & EXIT
;
DOSCMD	LD	SP,STACK	;reset stack
	PUSH	HL		;save it
	LD	HL,RESUME	;return vector
	EX	(SP),HL 	;leave on stack, get HL
;
;	$RETCMD - Interpret cmd & return
;
;	END	HL =>	command line
;
RTCMD0	LD	(DATAK5),HL	;save pointer
RETCMD	CALL	POSHL		;position to input
	RET	Z		;nil, return (Z set)
	CALL	REGSAV		;save the registers
	LD	DE,RUNDCB	;system DCB
	CALL	FSPEC		;move spec to DCB
	INC	DE		;adjust to normal DCB
	JR	NZ,GOERR	;post error
;
	PUSH	HL		;SAVE => cmd line
	LD	HL,LIBRARY	;library lookup table
	CALL	FIND		;library command?
	POP	HL		;GET => cmd line
	JR	NZ,GOTLIB	;have library command
;
;	Execute program file
;
	LD	A,(DE)		;check for device
	CP	'@'             ;device specifier?
	LD	A,NDRV		;'no drive available'
	JR	Z,GOERR 	;post error
;
;	OK, add default /CMD extension
;
	PUSH	HL		;save pointer
	LD	HL,CMD		;/CMD extension
	CALL	FEXT		;add extension
	POP	HL		;restore string pointer
	LD	A,-1		;disable high overlay
	LD	(OVLYH),A	;save it
	LD	A,RUN		;run program SVC #
	JR	GOSVC		;run it!
;
;	Execute library command
;
GOTLIB	LD	DE,LOOKUP-1	;lookup table
	ADD	A,E		;add entry number
	LD	E,A		;table on a page
	LD	A,(DE)		;get the command #
	RST	OVLY		;go!
;
;	Post error message
;
GOERR	AND	7FH		;no abort
	LD	B,A		;B = CODE
	LD	A,ERROR 	;SVC #
GOSVC	JP	SVC		;display it
;
	PAGE
;	Finds string in array routine
;
;	ENT	HL =>	TABLE ARRAY
;		DE =>	COMPARE STRING
;
FIND	LD	B,0		;position counter
FINLP	INC	B		;bump command #
	LD	A,(HL)		;GET table
	INC	HL		;next
	OR	A		;end list?
	JP	P,FINLP+1	;til 1st byte
	AND	7FH		;END?
	RET	Z		;not found!
;
	PUSH	DE		;save string
FINLP1	LD	C,A		;C = CHAR
	LD	A,(DE)		;get string byte
	CALL	UCASE		;make it upper
	CP	C		;same?
	JR	NZ,HAVCNT	;if not
	LD	A,(HL)		;get next char
	INC	DE		;bump string
	INC	HL		;next
	JR	FINLP1		;til no match
;
HAVCNT	POP	DE		;restore string
	BIT	7,C		;end table entry?
	JR	Z,FINLP 	;if not
	CP	3		;terminator?
	DEC	HL		;put pointer back
	JR	NZ,FINLP	;if yes
	LD	A,B		;get table posit
	OR	A		;set NZ flag
	RET			;done
;
	PAGE
;	Position HL to first significant char
;
;	ENT	HL => string to parse
;
;	EXT	HL => first significant char
;		C flag = HL => terminator (13,3,;)
;		NC flag = next char valid
;		A = character
;		Z flag always set
;		spaces and commas are ignored
;
	INC	HL		;bump pointer
POSHL	LD	A,(HL)		;get a byte
	CALL	CKSEP		;separator
	JR	Z,POSHL-1	;go if yes
;
CKTERM	CP	CR
	RET	Z
	CP	';'
	RET	Z
	CP	3
	RET
;
	PAGE
;	FEXT -add default extension to filename
;
;	ENT	DE => file DCB
;		HL => extension (3 chars)
;
;	EXT	HL, DE unchanged
;		Z = no extension added (one exists)
;		NZ = extension added
;
FEXT	CALL	REGSAV		;save reg
	LD	B,9		;max chars to check
	EX	DE,HL		;HL => DCB
;
FEXTLP	LD	A,(HL)		;get a character
	CP	'/'             ;extension now?
	JR	Z,RETURN	;not added
	CALL	VALCHAR 	;valid character?
	JR	Z,FEXTADD	;nope, add the extension
	INC	HL		;bump pointer
	DJNZ	FEXTLP		;check 9 chars
RETURN	XOR	A		;not added
	RET
;
FEXTADD PUSH	DE		;save extension
	LD	BC,24		;max length of ext+term
	ADD	HL,BC		;point to end
	LD	D,H		;give to DE
	LD	E,L
	INC	DE		;add 4 for /ext
	INC	DE
	INC	DE
	INC	DE
	INC	BC		;+ terminator
	LDDR			;make a hole
	POP	HL		;get ext pointer
	INC	HL		;point to end
	INC	HL
	LD	C,3		;BC = 3
	LDDR			;move in extension
	LD	A,'/'           ;put in /
	LD	(DE),A		;do it
	OR	A		;set NZ
	RET
;
	PAGE
;	Check for valid filespec character
;
;	ENT	A = char to check
;
;	EXT	NZ = char OK
;		Z = invalid char
;
VALCHAR PUSH	BC		;SAVE
	PUSH	HL		;save
	LD	HL,CHARTBL	;character table
	LD	BC,16		;COUNT
	CPIR			;FIND!
	POP	HL		;RESTORE
	POP	BC
	RET
;
;	Table of reserved characters
;
CHARTBL	DEFM	'('
	DEFM	')'
	DEFM	','
	DEFM	'.'
	DEFM	'/'
	DEFM	';'
	DEFM	' '
	DEFM	':'
	DEFM	'@'
	DEFM	'='
	DEFM	'"'
	DEFB	27H
	DB	ETX
	DB	CR
	DB	7BH		;braces
	DB	7DH
;
	PAGE
;	$FSPEC - fetch filespec into DCB
;
;	ENT	HL => ascii string
;		DE => 40 byte DCB
;
;	EXT	HL => terminating character
;		DE => DCB (41 bytes long)
;		NZ = A = error code, HL unchanged
;		Z = OK, HL=> terminating character
;
;	DCB returned:
;	+0	7 - device field evaluated
;		6 - filespec field evaluated
;		5 - filespec has wildcard characters
;		4-0 - device number 0-15 (global default)
;
;	+1->   filename, terminated with 03H (normal DCB)
;
FSPEC	PUSH	HL		;save string start
	PUSH	BC		;save this
	PUSH	DE		;save DCB start
;
;	Start initializing flags
;
	LD	A,1FH		;device number (nil)
	LD	(DE),A		;put it in
	INC	DE		;bump pointer to DCB
;
;	Fetch FILENAME field
;
	DEC	HL		;adjust filed
	CALL	MOVBLK8 	;move 8 bytes
	JP	NZ,CKDEV	;not a filespec
	JR	NC,MBLK1	;no wild chars
;
	EX	(SP),HL 	;get DCB first byte
	SET	5,(HL)		;set wild card bit
	EX	(SP),HL 	;put it back
;
MBLK1	EX	(SP),HL 	;get first
	SET	6,(HL)		;set contains file data
	EX	(SP),HL 	;put it back
;
;	Fetch EXTENSION field
;
	CP	'/'             ;start extension?
	JR	NZ,FSPEC1	;check password
	LD	(DE),A		;put in buffer
	INC	DE		;bump pointer
	LD	B,3		;3 char extension
	CALL	MOVBLK-1	;move it in
	JR	NZ,FSPCER	;invalid
	JR	NC,FSPEC1	;go if no wild cards
;
	EX	(SP),HL 	;get DCB start
	SET	5,(HL)		;turn on wild bit
	EX	(SP),HL 	;put it back
;
;	Fetch PASSWORD field
;
FSPEC1	CP	'.'             ;start password
	JR	NZ,FSPEC2	;check drive
	LD	(DE),A		;put it in
	INC	DE		;bump ptr
	CALL	MOVBLK8 	;move 8 bytes
	JR	NZ,FSPCER	;invalid
	JR	C,FSPCER	;wild card chars
;
;	Fetch DRIVE field
;
FSPEC2	CP	':'             ;drive number?
	JR	NZ,FSPEC3	;check for diskname
;
;	check for wild cards & device
;
	EX	(SP),HL 	;get flags
	BIT	5,(HL)		;wild bit?
	EX	(SP),HL 	;put it back
	JR	NZ,FSPEC5	;terminate right here
	INC	HL		;restore pointer
	LD	(DE),A		;put it in
	INC	DE		;bump ptr
	CALL	FDRV		;get the device number
	JR	NZ,FSPCER1	;invalid
;
	EX	(SP),HL 	;get start of DCB
	LD	B,A		;save it
	LD	A,(HL)		;get the byte
	AND	60H		;bits 6&5 only wanted
	OR	B		;merge it
	OR	80H		;set device evaluated
	LD	(HL),A		;update
	EX	(SP),HL 	;put it back
;
	LD	B,2		;move in the 2 chars
	CALL	MOVBLK		;move it in
;
;	Fetch DISKNAME field
;
FSPEC3	CP	'('             ;diskname?
	JR	NZ,FSPEC5	;terminate field
	LD	(DE),A		;put in buffer
	INC	DE		;bump ptr
;
	EX	(SP),HL 	;get start pointer
	BIT	7,(HL)		;device specified?
	EX	(SP),HL 	;put it back
	LD	A,32		;'invalid drivespec'
	JR	Z,FSPCER1	;invalid
	CALL	MOVBLK8 	;go for 8 chars
	JR	NZ,FSPCER	;invalid
	JR	C,FSPCER	;wild card chars
	CP	')'             ;closer?
	JR	NZ,FSPCER	;invalid
	LD	(DE),A		;put in buffer
	INC	DE		;bump ptr
	INC	HL		;bump
;
;	Terminate filespec
;
FSPEC5	LD	A,ETX		;terminator
	LD	(DE),A		;put it in
	POP	DE		;get DCB start
	POP	BC		;get this back
	POP	AF		;ignore string
	XOR	A		;set Z flag for OK
	RET			;return with Z
;
FSPCER	LD	A,19		;bad filespec
FSPCER1 POP	DE		;unstack
	POP	BC
	POP	HL		;unchanged
	OR	A		;set NZ flag
	RET			;done, A = error
;
;	Check for DEVICE specification
;
CKDEV	CP	':'             ;drive only?
	JR	Z,FSPEC2	;evaluate it
	CP	'@'             ;device specifier?
	JR	NZ,FSPCER	;go error
	LD	(DE),A		;save char
	INC	DE		;next
	INC	HL
	CALL	FDEV		;get device number
	JR	NZ,FSPCER1	;invalid
;
	EX	(SP),HL 	;get DCB start
	OR	80H		;set device evaluated
	LD	(HL),A		;put in device #
	EX	(SP),HL 	;put it back
;
	LD	B,2		;2 chars in name
	CALL	MOVBLK		;move into DCB
	JR	FSPEC5		;done
;
	PAGE
;	Extract field from master string
;
;	ENT	 B =	MAX FIELD LENGTH
;		HL =>	MASTER STRING
;		DE =>	FIELD DESTINATION
;
;	EXT	 A =	TERM CHAR
;		 Z SET IF FIELD FOUND
;		 C SET IF WILD CHARS
;
MOVBLK8 LD	B,8		;go for 8 chars
;
	INC	HL		;bump to next field
MOVBLK	LD	A,-1		;set error
	OR	A		;NC,NZ
	EX	AF,AF'          ;save flags
;
MOVBLK1 LD	A,(HL)		;get a byte
	CALL	VALCHAR 	;valid character?
	JR	Z,MOVBLK5	;invalid char
	INC	HL		;bump pointer
;
	CP	'?'             ;wild?
	JR	Z,MOVBLK2	;if yes
	CP	'!'		;wild?
	JR	Z,MOVBLK2	;yes, go!
	CP	'*'             ;wild?
	JR	NZ,MOVBLK3	;if not
;
MOVBLK2 EX	AF,AF'          ;get flags
	SCF			;have wild!
	EX	AF,AF'          ;save flags
;
MOVBLK3 LD	(DE),A		;put in buffer
	INC	DE		;bump pointer
	EX	AF,AF'          ;get flags back
	LD	A,0		;load with 0
	INC	A		;set Z, not C
	DEC	A
	EX	AF,AF'          ;save flag byte
	DJNZ	MOVBLK1 	;go more
;
;MOVBLK4 LD	A,(HL)		;fetch next char
;	CALL	CKTERM		;terminator?
;	JR	Z,MOVBLK5	;skip
;	CALL	CKSEP		;seperator?
;	JR	Z,MOVBLK5	;skip
;	INC	HL		;bump pointer
;
MOVBLK5	EX	AF,AF'          ;get flags
	LD	A,(HL)		;get term
	RET			;tats all fowks!
;
	PAGE
;	$FDRV - fetch drive spec from string
;
;	ENT	HL => string (1 or 2 bytes long)
;
;	EXT	HL unchanged
;		NZ = invalid or drive not found
;		Z = A = device number
;
FDRV	CALL	FINDEV		;look for the device
	JR	NZ,FDRVBAD	;no good
	CP	7		;must be 8-15
	JR	C,FDRVBAD	;no good
	CP	A		;set Z
	RET
;
FDRVBAD LD	A,32		;invalid drivespec
	OR	A
	RET			;done, NZ
;
;	FDEV - fetch device spec from string
;
;	ENT	HL => string (1 or 2 chars)
;
;	EXT	HL unchanged
;		NZ = A = error code (invalid or not found
;		Z  = A = device number
;
FDEV	CALL	FINDEV		;look for the device
	JR	NZ,FDEVBAD	;error
	CP	8		;must be 0-7
	JR	NC,FDEVBAD	;error
	CP	A		;set Z
	RET			;done
;
FDEVBAD LD	A,68		;invalid devicespec
	OR	A
	RET			;return with NZ
;
	PAGE
;	Find device/drive routine
;
;	ENT	HL => string (1 or 2 chars)
;
;	EXT	HL unchanged
;		NZ = A = error code, invalid or not found
;		Z  = A = device number
;
FINDEV	CALL	REGSAV		;SAVE REGISTERS
	LD	C,0		;C=device #
	LD	DE,4		;offset
	LD	IX,DCBTBL	;start of device names
;
FINDVLP LD	A,(HL)		;get string char
	CALL	UCASE		;make it upper
	CP	(IX+2)		;compare to table
	INC	HL		;next
	JR	NZ,FINDSKP	;check next device
	LD	A,(IX+3)	;get second char
	CP	' '             ;1 char?
	LD	B,A		;save char
	LD	A,(HL)		;get string char
	JR	Z,FINDOK1	;yes, go!
	CALL	UCASE		;make it upper
	CP	B		;match?
	JR	NZ,FINDSKP	;nope, next one
FINDOK	LD	A,C		;get device
	RET			;not valid,found!
;
FINDOK1	CALL	VALCHAR		;valid?
	JR	Z,FINDOK	;yes, go!
;
FINDSKP ADD	IX,DE		;next device
	DEC	HL		;1st char
	LD	A,15		;last
	INC	C		;next
	CP	C		;done?
	JR	NC,FINDVLP	;if not
	LD	A,68		;'invalid devicespec'
	RET
;
	PAGE
;	$PARAM - evaluate parameter list
;
;	ENT	HL => string to parse
;		DE => parameter list
;
;	EXT	DE is unchanged
;		Z = all parameters OK
;		NZ = A = error
;		HL => terminating/offending character
;
;	PARAMETER LIST:
;
;	A string of parameter blocks terminated with 00
;
;	PARAMETER BLOCK:
;
;	+0    - 7 - string expression can be evaluated
;		6 - value expression valid
;		5 - switch expression valid
;		4-0 - string length-1 (1-16 chars)
;	+1,+2 - parameter data indirect address
;	+3    - string (must be upper case)
;
;	(note) table chars must be in upper case, but
;	the input string evaluated is case independent
;
;	(note) if string is found with no expression
;	true (FFFF) is assumed.  In this case,
;	the switch valid bit MUST be set or an
;	error will be generated.
;
	INC	HL
PARAMB	XOR	A
	RET
PARAM	CALL	POSHL		;get next byte
	JR	Z,PARAMB
	CP	7BH		;param starter?
	JR	NZ,PARAM0	;go if not
	INC	HL		;bump pointer if yes
;
;	Check for parameter end
;
PARAM0	CALL	POSHL		;position pointer
	JR	Z,PARAMB
	CP	7DH		;param terminator?
	JR	Z,PARAMB-1
	PUSH	DE		;save param block
	CALL	DOPARAM 	;execute parameter
	POP	DE		;restore stack
	RET	NZ		;param not found
	JR	PARAM0		;else get next one
;
;	Evaluate parameter
;
DOPARAM LD	A,(DE)		;get length byte
	OR	A		;0 terminator?
	JR	Z,EXPERR	;go if yes
	AND	0FH		;low 4 bits only
	INC	A		;plus 1 for length
	LD	B,A		;give here for test
	PUSH	BC		;save length
	PUSH	DE		;save pointer
	CALL	THISONE 	;this entry?
	POP	DE		;restore
	POP	BC		;get length
	JR	Z,HAVTHIS	;get params
	INC	DE		;bump past vector
	INC	DE
	INC	DE
HAVNXT	INC	DE		;bump again
	DJNZ	HAVNXT		;position to next one
	JR	DOPARAM 	;check next word
;
;	Check for any expression
;
HAVTHIS LD	BC,-1		;default value to use
	LD	A,(HL)		;get next byte
	CP	'='             ;expression following?
	JR	Z,HAVEXP	;have an expression, go!
	EX	DE,HL		;get flags
	BIT	5,(HL)		;switch valid?
	EX	DE,HL		;put it back
	JR	Z,EXPERR	;expression error
;
;	Place result of expression
;
PUTPAR	PUSH	HL		;save pointer
	PUSH	DE		;this too
	INC	DE		;point to vector
	LD	A,(DE)		;get vector lsb
	LD	L,A		;put it here
	INC	DE		;point to MSB
	LD	A,(DE)		;get it
	LD	H,A		;HL => vector
	LD	(HL),C		;put value here
	INC	HL
	LD	(HL),B		;BC = param value
	POP	DE		;unstack
	POP	HL
	XOR	A		;set Z flag for OK
	RET
;
;	Check for string expression
;
HAVEXP	INC	HL		;bump string
	LD	A,(HL)		;get the byte
	CP	'"'             ;string?
	JR	Z,HAVSTR	;go!
	CP	27H		;' string?
	JR	Z,HAVSTR	;go!
	EX	DE,HL		;HL => param byte
	BIT	5,(HL)		;can it have switch
	EX	DE,HL		;put back
	JR	Z,EXPVAL	;no switch allowed
;
;	Evaluate logical expression
;
	PUSH	DE		;save table
	LD	DE,ONMSG-3	;on?
	LD	B,2		;2 chars
	CALL	THISONE 	;yes?
	JR	Z,HAVTRUE	;have a true
	LD	DE,OFFMSG-3	;off?
	LD	B,3
	CALL	THISONE 	;yes?
	JR	Z,HAVFAL	;have a false
	LD	DE,YESMSG-3	;yes?
	LD	B,3
	CALL	THISONE 	;yes?
	JR	Z,HAVTRUE	;have a true
	LD	DE,NOMSG-3	;no?
	LD	B,2
	CALL	THISONE 	;yes?
	JR	Z,HAVFAL	;have a false
	POP	DE		;restore it
;
;	Evaluate numeric expression
;
EXPVAL	EX	DE,HL		;get control byte
	BIT	6,(HL)		;can it have a value?
	EX	DE,HL
	JR	NZ,EXPVOK	;yes, go
EXPERR	LD	A,3		;param error
	OR	A
	RET
;
EXPVOK	CALL	VALUE		;get the value
	RET	NZ		;error
	JR	PUTPAR		;insert value
;
;	Evaluate string expression
;
HAVSTR	EX	DE,HL		;exchange
	BIT	7,(HL)		;string OK
	EX	DE,HL
	JR	Z,EXPERR	;expression error
	INC	HL		;bump past "
	LD	B,H		;BC => string vector
	LD	C,L
HAVSLP	LD	A,(HL)		;get next byte
	CALL	CKTERM		;terminator?
	JR	Z,PUTPAR	;put parameter
	CP	7DH		;close brace?
	JR	Z,PUTPAR
	INC	HL		;bump pointer
	CP	'"'             ;terminator?
	JR	Z,PUTPAR
	CP	27H		;terminator?
	JR	Z,PUTPAR	;go!
	JR	HAVSLP		;go till one found
;
HAVFAL	LD	BC,0		;0 = false
	POP	DE		;restore stack
	JR	PUTPAR		;put in table
;
HAVTRUE LD	BC,-1		;-1 = true
	JR	HAVFAL+3	;save a byte
;
;	Scan field for match
;
THISONE PUSH	HL		;save string pointer
	INC	DE		;bump to string
	INC	DE		;past vector
	INC	DE
	XOR	A		;init count
	EX	AF,AF'          ;save it here
THISLP	LD	A,(HL)		;get a byte
	CALL	UCASE		;make it upper case
	LD	C,A		;save it here
	LD	A,(DE)		;get param byte
	CP	C		;same?
	JR	NZ,THISNOT	;nope
	INC	HL		;bump pointers
	INC	DE		;this too
	EX	AF,AF'          ;get flag byte
	INC	A		;bump match count
	EX	AF,AF'          ;put it back
	DJNZ	THISLP		;do whole thing
	EX	(SP),HL 	;replace new HL
	POP	HL		;unstack
	EX	AF,AF'          ;get counter back
	CP	A		;set Z
	RET
;
THISNOT EX	AF,AF'          ;get flag
	DEC	A		;1 byte matched?
	JR	NZ,THISBAD	;not valid, original HL
	LD	A,(HL)		;get last byte
	CALL	VALCHAR 	;must be sep/term
	JR	NZ,THISBAD	;valid char!
	EX	(SP),HL 	;else put this one
	POP	HL		;unstack
	LD	A,1		;set 1 byte match
	CP	A		;set Z
	RET			;done, Z = OK
;
THISBAD POP	HL		;unstack
	OR	A		;NZ
	RET
;
	PAGE
;	$VALUE - fetch value from string
;
;	ENT	HL => ascii string
;
;	EXT	Z = OK, BC = value
;		NZ = A = error
;		HL => string terminator
;
;	Number can be hex, octal, binary, decimal(default
;	by appending H, O, B or D to the end
;
VALUE	PUSH	HL		;save string pointer
	CALL	POSEND		;point to end byte
	LD	HL,ADDHEX	;hex adder
	LD	B,16		;max digits
	CP	'H'             ;hex specified?
	JR	Z,GOVAL 	;go if yes
	LD	HL,ADDOCT	;octal adder
	LD	B,8		;max
	CP	'O'             ;oct specified?
	JR	Z,GOVAL 	;go if yes
	CP	'Q'		;oct?
	JR	Z,GOVAL		;go if yes
	LD	HL,ADDBIN	;binary adder
	LD	B,2		;max
	CP	'B'             ;bin specified?
	JR	Z,GOVAL 	;go if yes
	LD	HL,ADDDEC	;decimal adder
	LD	B,10		;max
	CP	'D'             ;decimal?
	JR	Z,GOVAL 	;go if yes
	XOR	A		;else nil special term
GOVAL	LD	(ADDCALL),HL	;put adder call to code
	LD	(ADDTERM),A	;put alt term char here
	POP	HL		;get string back
	LD	A,B		;get max digit
	LD	(ADDMAX),A	;put in code
	LD	BC,0		;start value
VALLP	LD	A,(HL)		;get a byte
	CALL	UCASE		;make it upper
	CALL	CKTERM		;terminator
	JR	Z,VALOK		;OK
	CP	7DH		;term brace?
	JR	Z,VALOK
	INC	HL		;bump pointer
	CP	0		;base terminator?
ADDTERM EQU	$-1
	JR	Z,VALOK		;go if yes
	CALL	CKSEP		;separator?
	JR	Z,VALOK		;go if yes
;
;	Actual conversion here
;
	CALL	CONVCHR 	;convert character
	RET	NZ		;parameter error
	PUSH	HL		;save pointer
	LD	H,B		;get subtotal
	LD	L,C
	ADD	HL,HL		;start with 2*
	CALL	0		;multiple subtotal
ADDCALL EQU	$-2		;set from above
	LD	B,H		;put it back
	LD	C,L
	POP	HL		;unstack
	ADD	A,C		;add new byte
	LD	C,A		;put it back
	JR	NC,VALLP	;go more
	INC	B		;bump msb
	JR	VALLP		;go next byte
;
VALOK	XOR	A
	RET			;done, go back
;
;	Convert digit to binary
;
CONVCHR SUB	'0'             ;remove ascii
	JR	C,CHBAD 	;invalid digit
	CP	10		;0-9?
	JR	C,CHOK		;go if ok
	SUB	7		;adjust for A-F
	JR	C,CHBAD 	;invalid
CHOK	CP	0		;test for max digit
ADDMAX	EQU	$-1
	JR	NC,CHBAD	;invalid
	CP	A		;set Z for OK
	RET			;done
;
CHBAD	LD	A,3		;parameter error
	OR	A		;set NZ
	RET			;done
;
;	Hex/Octal conversions
;
ADDHEX	ADD	HL,HL		;*16
ADDOCT	ADD	HL,HL		;*8
	ADD	HL,HL		;*4
	RET			;done
;
;	Decimal/Binary conversions
;
ADDDEC	ADD	HL,HL		;*2
	ADD	HL,BC		;*5
	ADD	HL,HL		;*2more
ADDBIN	RET			;HL = product
;
;	Posn to end numeric field
;
	INC	HL		;bump pointer
POSEND	LD	A,(HL)		;get a char
	CALL	CKTERM		;terminator?
	JR	Z,ENDHAV	;have it
	CP	7DH		;right brace?
	JR	Z,ENDHAV	;have end
	CALL	CKSEP		;separator?
	JR	NZ,POSEND-1	;nope, continue
;
ENDHAV	DEC	HL		;put it back
	LD	A,(HL)		;get it
	JP	UCASE		;convert and return
;
	PAGE
;	$EVAL - evaluate source,dest,mask,params
;
;	ENT  :	IX => parameter block
;		HL => string to evaluate
;
;	EXT :	HL => terminating/offending character
;		DE => param address for PARAM (if Z set)
;		Z  = evaluation OK
;		NZ = A = error code (DE MAY be unchanged)
;		if Z, parameter blocks are loaded
;
;	PARAMETER BLOCK:
;
;	+0    - work byte for flag use
;	+1,+2 - source DCB pointer (41 bytes)
;	+3,+4 - dest DCB pointer (41 bytes)
;	+5,+6 - mask DCB pointer (41 bytes)
;	+7,+8 - parameter list address for PARAM
;
;	On exit, IX+0:
;	bit 2 = 1 = source field filled
;	bit 1 = 1 = dest field filled
;	bit 0 = 1 = mask field filled
;
EVAL	PUSH	BC		;save these
	LD	(IX),0		;reset flags
;
EVALLP	LD	A,(IX)		;get the flag
	AND	7		;low 3 bits only
	LD	(IX),A		;re-save
;
;	Check for PARAM field
;
EVAL0	LD	A,(HL)		;get a byte
	CP	','             ;param field broken?
	JR	Z,EVALP 	;go param
	CP	7BH		;param field?
	JR	Z,EVALP 	;go param
;
;	Check for field end
;
	CALL	CKTERM		;terminator
	JR	Z,EVALRET	;done
;
;	Check for field tabs
;
	INC	HL		;bump pointer
	CP	20H		;separator?
	JR	Z,EVAL0 	;go if yes
	DEC	HL		;less 1
;
;	TO field specifier?
;
	LD	DE,TOMSG-3	;to?
	LD	B,2
	CALL	THISONE 	;yes?
	JR	NZ,CKFROM	;no, go
	DEC	A		;one char match?
	JR	Z,CKFROM-1	;skip if yes
	SET	6,(IX)		;set TO flag
	JR	EVAL0		;position to next one
;
EVALERR LD	A,3		;parameter error
	OR	A		;set NZ
;
;	Exit EVAL routine here
;
EVALRET POP	BC
	RET	NZ
	XOR	A
	RET
;
;	Evaluate PARAM field
;
EVALP	POP	BC		;unstack
	LD	E,(IX+7)	;get parameter block
	LD	D,(IX+8)
	JP	PARAM		;go param
;
;	FROM field specifier?
;
	DEC	HL		;reset pointer
CKFROM	LD	DE,FROMMSG-3	;from?
	LD	B,4		;length
	CALL	THISONE
	JR	NZ,CKUSE	;check using
	DEC	A		;one char match only?
	JR	Z,CKUSE-1	;backspace & ignore
	SET	7,(IX)		;set FROM flag
	JR	EVAL0		;go next word
;
;	USING field specifier?
;
	DEC	HL		;backspace pointer
CKUSE	LD	DE,USEMSG-3	;using?
	LD	B,5
	CALL	THISONE
	JR	NZ,CKNXT	;no, go
	DEC	A		;one char only?
	JR	Z,CKNXT-1	;ignore if yes
	SET	5,(IX)		;set from
	JR	EVAL0		;go next word
;
;	Fetch field here
;
	DEC	HL		;backspace
CKNXT	LD	DE,BUFFER	;work area
	CALL	FSPEC		;evaluate device
	JR	NZ,EVALRET	;invalid spec
	LD	A,(DE)		;get type byte
	BIT	5,A		;wild cards?
	JR	NZ,CHKMASK	;wild, force MASK
	LD	A,(IX)		;get the byte
	AND	0E0H		;to from using?
	JR	NZ,EVALC	;go if yes
	LD	A,(IX)		;get again
	CPL			;reverse the bits
	RLCA
	RLCA			;align to high bits
	RLCA
	RLCA
	RLCA
;
EVALC	RLCA			;check bit 7
	JR	C,FILSRC	;go source
	RLCA
	JR	C,FILDES
	JR	FILMASK 	;go fill mask
;
CHKMASK LD	A,(IX)		;see if TO FROM ?
	AND	0C0H		;yes?
	JR	NZ,EVALERR	;error if yes
;
;	Fill MASK field
;
FILMASK BIT	0,(IX)		;mask filled?
	JR	NZ,EVALERR	;return if yes
	LD	C,(IX+5)	;else fill mask
	LD	B,(IX+6)
	SET	0,(IX)		;set filled
	JR	EVALFIL 	;fill the buffer
;
;	Fill SOURCE field
;
FILSRC	BIT	2,(IX)		;source filled?
	JR	NZ,EVALERR	;yes, return
	LD	C,(IX+1)	;get source addr
	LD	B,(IX+2)
	SET	2,(IX)		;set filled
	JR	EVALFIL 	;fill it
;
;	Fill DESTINATION field
;
FILDES	BIT	1,(IX)		;dest filled?
	JP	NZ,EVALERR	;yes, return
	LD	C,(IX+3)	;get addr
	LD	B,(IX+4)
	SET	1,(IX)		;set filled
;
;	Fill GENERAL field
;
EVALFIL PUSH	HL		;save it
	EX	DE,HL		;HL => DCB
	LD	D,B		;load DE with BC
	LD	E,C
	LD	BC,41		;length
	LDIR			;move to correct area
	POP	HL		;get string back
	JP	EVALLP		;get next char
;
;	Check for separator character
;
CKSEP	CP	' '             ;space?
	RET	Z		;go if yes
	CP	','             ;comma?
	RET			;Z = separator
;
	PAGE
;	$LIB - library command
;
LIB	LD	HL,LIBRARY	;library command table
	RET			;back to caller
;
;	$CLS	- clear video screen
;
CLS	LD	B,1BH		;clear screen code
	LD	A,VDCHAR	;SVC #
	RST	SVC		;clear screen
	XOR	A		;force ZERO return
	RET			;done, Z = result
;
;	$BOOT	- cold reset the system
;
BOOT	LD	A,1
	OUT	(0F9H),A
	RST	0
;
;	Messages and text strings
;
PROMPT	DB	CR		;next line
	DB	ETB		;clear line
	DEFM	'DosPLUS II'
	DB	CR		;next line
PRLEN	EQU	$-PROMPT
;
FROMMSG DEFM	'FROM'
USEMSG	DEFM	'USING'
CMD	DEFM	'CMD'
YESMSG	DEFM	'YES'
TOMSG	DEFM	'T'
ONMSG	DEFM	'ON'
NOMSG	EQU	$-1
OFFMSG	DEFM	'OFF'
;
	PAGE
;	Library command lookup table
;	Each byte is a RST 30H command
;
LOOKUP	DB	10-1<4+2	;append
	DB	14-1<4+2	;attrib
	DB	12-1<4+8	;auto
	DB	1-1<4+4		;boot
	DB	15-1<4+4	;build
	DB	9-1<4+1 	;cat
	DB	15-1<4+1	;clear
	DB	12-1<4+9	;clock
	DB	1-1<4+15	;cls
	DB	16-1<4+2	;config
	DB	10-1<4+1	;copy
	DB	15-1<4+2	;create
	DB	12-1<4+6	;date
	DB	12-1<4+5	;debug
	DB	9-1<4+2 	;dir
	DB	13-1<4+2	;do
	DB	15-1<4+3	;dump
	DB	12-1<4+4	;error
	DB	11-1<4+5	;filter
	DB	13-1<4+3	;forms
	DB	9-1<4+3		;free
	DB	12-1<4+3	;i
	DB	14-1<4+3	;kill
	DB	12-1<4+10	;lib
	DB	11-1<4+4	;link
	DB	10-1<4+3	;list
	DB	15-1<4+5	;load
	DB	12-1<4+11	;pause
	DB	14-1<4+1	;prot
	DB	15-1<4+6	;rename
	DB	11-1<4+2	;reset
	DB	11-1<4+1	;route
	DB	12-1<4+12	;screen
	DB	11-1<4+3	;set
	DB	13-1<4+1	;setcom
	DB	16-1<4+1	;system
	DB	12-1<4+2	;time
	DB	12-1<4+1	;verify
;
	PAGE
;	Table of library words
;
LIBRARY DEFB	'A'+80H         ;append
	DEFM	'PPEND'
	DEFB	'A'+80H         ;attrib
	DEFM	'TTRIB'
	DEFB	'A'+80H         ;auto
	DEFM	'UTO'
	DEFB	'B'+80H         ;boot
	DEFM	'OOT'
	DEFB	'B'+80H         ;build
	DEFM	'UILD'
	DEFB	'C'+80H         ;cat
	DEFM	'AT'
	DEFB	'C'+80H         ;clear
	DEFM	'LEAR'
	DEFB	'C'+80H         ;clock
	DEFM	'LOCK'
	DEFB	'C'+80H         ;cls
	DEFM	'LS'
	DEFB	'C'+80H         ;config
	DEFM	'ONFIG'
	DEFB	'C'+80H         ;copy
	DEFM	'OPY'
	DEFB	'C'+80H         ;create
	DEFM	'REATE'
	DEFB	'D'+80H         ;date
	DEFM	'ATE'
	DEFB	'D'+80H         ;debug
	DEFM	'EBUG'
	DEFB	'D'+80H         ;dir
	DEFM	'IR'
	DEFB	'D'+80H         ;do
	DEFM	'O'
	DEFB	'D'+80H         ;dump
	DEFM	'UMP'
	DEFB	'E'+80H         ;error
	DEFM	'RROR'
	DEFB	'F'+80H         ;filter
	DEFM	'ILTER'
	DEFB	'F'+80H         ;forms
	DEFM	'ORMS'
	DEFB	'F'+80H         ;free
	DEFM	'REE'
	DEFB	'I'+80H         ;i
	DEFB	'K'+80H         ;kill
	DEFM	'ILL'
	DEFB	'L'+80H         ;lib
	DEFM	'IB'
	DEFB	'L'+80H         ;link
	DEFM	'INK'
	DEFB	'L'+80H         ;list
	DEFM	'IST'
	DEFB	'L'+80H         ;load
	DEFM	'OAD'
	DEFB	'P'+80H         ;pause
	DEFM	'AUSE'
	DEFB	'P'+80H         ;prot
	DEFM	'ROT'
	DEFB	'R'+80H         ;rename
	DEFM	'ENAME'
	DEFB	'R'+80H         ;reset
	DEFM	'ESET'
	DEFB	'R'+80H         ;route
	DEFM	'OUTE'
	DEFB	'S'+80H         ;screen
	DEFM	'CREEN'
	DEFB	'S'+80H         ;set
	DEFM	'ET'
	DEFB	'S'+80H         ;setcom
	DEFM	'ETCOM'
	DEFB	'S'+80H         ;system
	DEFM	'YSTEM'
	DEFB	'T'+80H         ;time
	DEFM	'IME'
	DEFB	'V'+80H         ;verify
	DEFM	'ERIFY'
	DEFB	80H
;
ZZZZZ	EQU	$
;
	END	VECTORS
